package controllers;

import play.*;
import play.mvc.*;

import java.util.*;

import models.*;

import uk.co.petertribble.jproc.server.JProcServer;

/**
 * JPplay exposes /proc  data in the RESTful format expected by the jproc
 * client, using the play framework. There is a simple and obvious mapping
 * between the URI and the method name.
 *
 * See the routes file, which maps a URI and its arguments to the methods here.
 *
 * This class is purely a wrapper of JProcServer, exposing exactly the same
 * methods just wrapped up in renderJSON for Play's use.
 */
public class JPplay extends Controller {

    public static JProcServer jps = new JProcServer();

    /**
     * .../getProcesses gives the list of processes
     */
    public static void getProcesses() {
	renderJSON(jps.getProcesses());
    }

    /**
     * .../getLwps gives the list of lwps for a process
     */
    public static void getLwps(String pid) {
	renderJSON(jps.getLwps(Integer.valueOf(pid)));
    }

    /**
     * .../getInfo gives the info for a process
     */
    public static void getInfo(String pid) {
	renderJSON(jps.getInfo(Integer.valueOf(pid)));
    }

    /**
     * .../getLwpInfo gives the info for a process lwp
     */
    public static void getLwpInfo(String pid, String lwpid) {
	renderJSON(jps.getLwpInfo(Integer.valueOf(pid),
				Integer.valueOf(lwpid)));
    }

    /**
     * .../getStatus gives the status for a process
     */
    public static void getStatus(String pid) {
	renderJSON(jps.getStatus(Integer.valueOf(pid)));
    }

    /**
     * .../getLwpStatus gives the status for a process lwp
     */
    public static void getLwpStatus(String pid, String lwpid) {
	renderJSON(jps.getLwpStatus(Integer.valueOf(pid),
				Integer.valueOf(lwpid)));
    }

    /**
     * .../getUsage gives the usage for a process
     */
    public static void getUsage(String pid) {
	renderJSON(jps.getUsage(Integer.valueOf(pid)));
    }

    /**
     * .../getLwpUsage gives the usage for a process lwp
     */
    public static void getLwpUsage(String pid, String lwpid) {
	renderJSON(jps.getLwpUsage(Integer.valueOf(pid),
				Integer.valueOf(lwpid)));
    }

    /**
     * .../getUserName gives the name for a given uid
     */
    public static void getUserName(String uid) {
	renderJSON(jps.getUserName(Integer.valueOf(uid)));
    }

    /**
     * .../getUserId gives the uid for a given name
     */
    public static void getUserId(String username) {
	renderJSON(jps.getUserId(username));
    }

    /**
     * .../getGroupName gives the name for a given gid
     */
    public static void getGroupName(String gid) {
	renderJSON(jps.getGroupName(Integer.valueOf(gid)));
    }

    /**
     * .../getGroupId gives the gid for a given name
     */
    public static void getGroupId(String group) {
	renderJSON(jps.getGroupId(group));
    }

    /**
     * .../getProjectName gives the name for a given gid
     */
    public static void getProjectName(String projid) {
	renderJSON(jps.getProjectName(Integer.valueOf(projid)));
    }

    /**
     * .../getProjectId gives the gid for a given name
     */
    public static void getProjectId(String project) {
	renderJSON(jps.getProjectId(project));
    }

    /**
     * .../getZoneName gives the name for a given uid
     */
    public static void getZoneName(String zoneid) {
	renderJSON(jps.getZoneName(Integer.valueOf(zoneid)));
    }

    /**
     * .../getZoneId gives the uid for a given name
     */
    public static void getZoneId(String zone) {
	renderJSON(jps.getZoneId(zone));
    }
}
